package com.gic.cloud.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.constants.LogAndUpdateTipsTypeEnum;
import com.gic.cloud.constants.UpdateTypeEnum;
import com.gic.cloud.dto.IndexDTO;
import com.gic.cloud.entity.TabIndex;
import com.gic.cloud.qo.IndexQO;
import com.gic.cloud.service.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-07-08 10:19
 */
@Service("indexApiService")
public class IndexApiServiceImpl implements IndexApiService {

    @Autowired
    private IndexService indexService;
    @Autowired
    private IndexLogService indexLogService;
    @Autowired
    private IndexModuleRelService indexModuleRelService;
    @Autowired
    private UpdateTipService updateTipService;

    @Override
    public ServiceResponse<Void> saveIndex(IndexDTO indexDTO) {
        // 校验参数
        if (indexService.isNameRepeat(indexDTO.getName(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
        }
        if (indexService.isCodeRepeat(indexDTO.getCode(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "编码重复");
        }
        // 保存实体
        Integer indexId = indexService.save(indexDTO);
        indexDTO.setIndexId(indexId);
        // 保存关联表
        indexModuleRelService.save(indexDTO.getIndexId(), indexDTO.getRelModuleIdList(), 1);
        // 写日志
        indexLogService.saveIndexInsertLog(indexDTO);

        // 红点提示
        updateTipService.insertBySaveBusiness(indexId, LogAndUpdateTipsTypeEnum.INDEX.getCode());

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateIndex(IndexDTO indexDTO, Integer updateType, String reason) {
        // 校验参数
        if (indexService.isNameRepeat(indexDTO.getName(), indexDTO.getIndexId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
        }
        if (indexService.isCodeRepeat(indexDTO.getCode(), indexDTO.getIndexId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "编码重复");
        }
        // 保存实体
        indexService.update(indexDTO);
        // 保存关联表
        indexModuleRelService.save(indexDTO.getIndexId(), indexDTO.getRelModuleIdList(), 1);

        if (UpdateTypeEnum.UPDATE_PUBLISH.getCode().equals(updateType)) {
            // 写日志
            indexLogService.saveIndexUpdateLog(indexDTO, reason);
            // 红点提示
            updateTipService.publish(indexDTO.getIndexId(), LogAndUpdateTipsTypeEnum.INDEX.getCode());
        }


        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<IndexDTO> getByIndexId(Integer indexId) {
        TabIndex index = this.indexService.getByIndexId(indexId);
        IndexDTO indexDTO = EntityUtil.changeEntityByJSON(IndexDTO.class, index);
        List<String> moduleIdList = this.indexModuleRelService.listModuleIdByBusinessId(indexDTO.getIndexId(), LogAndUpdateTipsTypeEnum.INDEX.getCode());
        indexDTO.setRelModuleIdList(moduleIdList);
        return EnterpriseServiceResponse.success(indexDTO);
    }

    @Override
    public ServiceResponse<Page<IndexDTO>> listIndex(IndexQO indexQO) {
        com.github.pagehelper.Page<TabIndex> indexList = indexService.listIndex(indexQO);
        Page<IndexDTO> indexDTOPage = PageHelperUtils.changePageHelperToCurrentPage(indexList, IndexDTO.class);
        if (CollectionUtils.isNotEmpty(indexList)) {
            List<Integer> indexIdList = indexList.stream().map(TabIndex::getIndexId).collect(Collectors.toList());
            Map<Integer, List<String>> moduleIdByBusinessIdMap = indexModuleRelService.groupModuleIdByBusinessIds(indexIdList, LogAndUpdateTipsTypeEnum.INDEX.getCode());
            List<IndexDTO> indexDTOList = indexDTOPage.getResult();
            for (IndexDTO indexDTO : indexDTOList) {
                indexDTO.setRelModuleIdList(moduleIdByBusinessIdMap.get(indexDTO.getIndexId()));
            }
        }
        return EnterpriseServiceResponse.success(indexDTOPage);
    }

    @Override
    public ServiceResponse<List<IndexDTO>> listAllIndex() {
        List<TabIndex> tabIndices = indexService.listAllIndex(new IndexQO());
        List<IndexDTO> dtoList = EntityUtil.changeEntityListByJSON(IndexDTO.class, tabIndices);
        return EnterpriseServiceResponse.success(dtoList);
    }

    @Override
    public ServiceResponse<Void> delete(Integer indexId) {
        TabIndex index = this.indexService.getByIndexId(indexId);
        if (index == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "指标不存在或已删除");
        }
        List<String> moduleIdList = indexModuleRelService.listModuleIdByBusinessId(indexId, LogAndUpdateTipsTypeEnum.INDEX.getCode());
        if (CollectionUtils.isNotEmpty(moduleIdList)) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不能删除被引用的指标");
        }
        this.indexService.deleteById(indexId);
        this.indexModuleRelService.deleteByBusinessId(indexId, LogAndUpdateTipsTypeEnum.INDEX.getCode());
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> publish(List<Integer> indexIdList, Integer updateType, String reason, Integer optUserId, String optUserName) {
        for (Integer indexId : indexIdList) {
            this.updateTipService.publish(indexId, LogAndUpdateTipsTypeEnum.INDEX.getCode());
            IndexDTO indexDTO = new IndexDTO();
            indexDTO.setIndexId(indexId);
            indexDTO.setOptUserId(optUserId);
            indexDTO.setOptUserName(optUserName);
            indexLogService.saveIndexUpdateLog(indexDTO, reason);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<IndexDTO>> listByModule(String moduleId) {
        IndexQO indexQO = new IndexQO();
        indexQO.setModuleId(moduleId);
        List<TabIndex> indexList = indexService.listAllIndex(indexQO);
        return EnterpriseServiceResponse.success(EntityUtil.changeEntityListByJSON(IndexDTO.class, indexList));
    }
}
